#include <iostream>
#include "AMQPcpp.h"

void process_message (AMQPMessage* message)
{
	cout << "message received with routing key: " << message->getRoutingKey() << endl;
	cout << message->getMessage() << endl;
}

int on_message (AMQPMessage* message)
{
	process_message(message);
	message->getQueue()->Ack();
	return 0;
}

int main (int argc, char* const argv[])
{
	try
	{
		AMQP api("localhost");
		AMQPExchange* exchange = api.createExchange();
		exchange->Declare("test_exchange", "topic", AMQP_DURABLE);
		
		AMQPQueue* queue = api.createQueue();
		queue->Declare("test_queue", AMQP_DURABLE);
		
		queue->Bind("test_exchange", "#");
		queue->addEvent(AMQP_MESSAGE, on_message);
		
		queue->setConsumerTag("qwerty");
		
		// messages are consumed (delivered down the channel), but never processed via the on_message callback function
		queue->Consume();
		
		// Get works fine (comment out Consume above)
		queue->Get();
		AMQPMessage* m = queue->getMessage();
		cout << "number of messages: " << m->getMessageCount() << endl;
		process_message(m);
		queue->Ack();
	}
	catch (AMQPException ex)
	{
		cout << "EXCEPTION OCCURRED!" << endl;
		cout << ex.getMessage() << endl;
	}
	
	return 0;
}